from itertools import starmap
import re


def subtitles_time_format(ms):
    """
    Formats subtitles time
    """
    seconds, milliseconds = divmod(ms, 1000)
    minutes, seconds = divmod(seconds, 60)
    hours, minutes = divmod(minutes, 60)
    return f'{hours:02}:{minutes:02}:{seconds:02},{milliseconds:02}'


def write_subtitles(count, subs, video_name, course_slug, chapter_name, video_duration):
    """
    Writes to a file(subtitle file) caption matching the right time
    """
    def subs_to_lines(idx, sub):
        starts_at = sub['transcriptStartAt']
        ends_at = subs[idx]['transcriptStartAt'] if idx < len(
            subs) else video_duration
        caption = sub['caption']
        return f"{idx}\n" \
            f"{subtitles_time_format(starts_at)} --> {subtitles_time_format(ends_at)}\n" \
            f"{caption}\n\n"

    with open(f"{course_slug}/{clean_name(chapter_name)}/{count}-{clean_name(video_name).strip()}.srt", 'wb') as f:
        for line in starmap(subs_to_lines, enumerate(subs, start=1)):
            f.write(line.encode('utf8'))

def clean_name(name):
    digit_removed = re.sub(r'^\d+\.', "", name)
    chars_removed = re.sub(r'[\\:<>"/|?*’.\')(,]', "", digit_removed).replace("«", " ").replace("-»", " ").replace("»", " ").strip()
    extra_space_removed= re.sub(r'(\s+)', " ", chars_removed)
    return extra_space_removed.strip()


# Don't change this, unless you know what you are doing :)
login_url = "https://www.linkedin.com/checkpoint/lg/login"
signup_url = "https://www.linkedin.com/checkpoint/lg/login-submit"
course_url = "https://www.linkedin.com/learning-api/detailedCourses??fields=videos&addParagraphsToTranscript=true&courseSlug={0}&q=slugs"
video_url = "https://www.linkedin.com/learning-api/detailedCourses?addParagraphsToTranscript=false&courseSlug={0}&q=slugs&resolution=_{1}&videoSlug={2}"



import sys
import click


def parse_cookie_file():
    try:
        with open("cookies.txt", "r") as file:
            cookies = {}
            for line in file:
                line = line.strip()
                if line.startswith("li_at"):
                    cookies["li_at"] = line.split("li_at=")[1]
                if line.startswith("JSESSIONID"):
                    cookies["JSESSIONID"] = line.split("JSESSIONID=")[1].replace(
                        '\"', "")
            return cookies
    except FileNotFoundError:
        click.echo(
            click.style(f"cookies.txt not found or is empty", fg="red"))
        sys.exit(0)




from tqdm import tqdm
import requests
import time
import click
import re
import os
from llvd.utils import clean_name


def download_video(url, index, filename, chapter_name, course_slug):
    """
        Downloads a video and saves it by its name plus index for easy sorting
    """
    print("\n" + clean_name(filename) + "\n")
    maximum_retries = 5
    with open(f'{course_slug}/{clean_name(chapter_name)}/{index}-{clean_name(filename)}.mp4', 'wb') as f:
        download_size = 0
        while maximum_retries > 0:
            requests.adapters.HTTPAdapter(max_retries=maximum_retries)
            response = requests.get(
                url, stream=True, headers={'Accept-Encoding': None, 'Content-Encoding': 'gzip'})
            download_size = response.headers.get('content-length')
            if download_size is None and maximum_retries > 0:
                maximum_retries -= 1
            else:
                break
        pbar = tqdm(
            total=int(download_size),
            initial=0,
            unit='B',
            unit_scale=True,
            position=0,
            leave=True)
        for chunk in response.iter_content(chunk_size=1024):
            if chunk:
                f.write(chunk)
                pbar.set_description("progress")
                pbar.update(len(chunk))
        pbar.close()
        print("\n")


def download_exercises(links, course_slug):
    """
        Downloads exercises
    """
    maximum_retries = 5
    click.echo(
        click.style(f"Downloading exercise files..." + "\n", fg="green"))

    for link in links:

        filename = re.split("exercises/(.+).zip", link)[1]

        with open(f"{course_slug}/{clean_name(filename)}.zip", 'wb') as f:
            download_size = 0
            while maximum_retries > 0:
                requests.adapters.HTTPAdapter(max_retries=maximum_retries)
                response = requests.get(
                    link, stream=True, headers={'Accept-Encoding': None, 'Content-Encoding': 'gzip'})
                download_size = response.headers.get('content-length')
                if download_size is None and maximum_retries > 0:
                    maximum_retries -= 1
                else:
                    break
            pbar = tqdm(
                total=int(download_size),
                initial=0,
                unit='B',
                unit_scale=True,
                position=0,
                leave=True)
            for chunk in response.iter_content(chunk_size=1024):
                if chunk:
                    f.write(chunk)
                    pbar.set_description("progress")
                    pbar.update(len(chunk))
            pbar.close()
            print("\n")



# !/usr/bin/env  python3

import click
import sys
import re
from llvd.app import App
from llvd import config
from llvd.process_io import parse_cookie_file


BOLD = "\033[1m"  # Makes the text bold
RED_COLOR = "\u001b[31m"  # Makes the text red


@click.command()
@click.option('--cookies', is_flag=True,
              prompt='Do you want to login with cookies?',
              help="Authenticate with cookies by following the guidelines provided in the documentation")
@click.option("--resolution", "-r",
              default='720',
              help='Video resolution can either be 360, 540 or 720. 720 is the default')
@click.option("--caption", "-ca",
              is_flag=True,
              help="Download subtitles")
@click.option("--course", "-c", help="Example: 'java-8-essential'")
def main(cookies, course, resolution, caption):
    """
    Linkedin learning video downloader cli tool

    example: llvd --course "java-8-essential"
    """
    if len(sys.argv) == 1:
        click.echo(f"{RED_COLOR}{BOLD}Missing required arguments: llvd --help")
        sys.exit(0)

    course = course.lower().replace("c#", "c-sharp").replace(".net", "-dot-net")
    without_chars = re.sub(r'[\':)(,>.’/]', " ", course.strip()).replace("«", " ").replace("-»", " ").replace("»", " ").strip()
    course_slug= re.sub(r'(\s+)', "-", without_chars.lower()).replace("--", "-")
    email = ""
    password = ""
    if cookies:
        cookie_dict = parse_cookie_file()
        if "li_at" not in cookie_dict or "JSESSIONID" not in cookie_dict:
            click.echo(
                click.style(f"cookies.txt must not be empty", fg="red"))
            sys.exit(0)

        llvd = App(email, password, course_slug, resolution, caption)
        llvd.run(cookie_dict)
    else:
        email = click.prompt("Please enter your Linkedin email address")
        password = click.prompt(
            "Enter your Linkedin Password", hide_input=True)

        llvd = App(email, password, course_slug, resolution, caption)
        llvd.run()



import os
import re
import sys
import requests
import click
from bs4 import BeautifulSoup as bs
from requests import Session
from llvd import config
from llvd.downloader import download_video, download_exercises
from click_spinner import spinner
import re
from llvd.utils import write_subtitles, clean_name


class App:
    def __init__(self, email, password, course_slug, resolution, caption):

        self.email = email
        self.password = password
        self.course_slug = course_slug
        self.link = ""
        self.video_format = resolution
        self.caption = caption
        self.headers = {}
        self.cookies = {}

    def login(self, s, login_data):
        """
            Login the user
        """
        with spinner():

            s.post(
                config.signup_url, login_data)
            cookies = s.cookies.get_dict()
            self.cookies["JSESSIONID"] = cookies.get("JSESSIONID").replace(
                '\"', "")
            self.cookies["li_at"] = cookies.get("li_at")
            self.headers["Csrf-Token"] = cookies.get("JSESSIONID").replace(
                '\"', "")

            if cookies.get("li_at") == None:
                return None
            return 200

    def run(self, cookies=None):
        """
        Main function, tries to login the user and when it succeeds, tries to download the course
        """
        try:

            if cookies is not None:
                self.cookies["JSESSIONID"] = cookies.get("JSESSIONID")
                self.cookies["li_at"] = cookies.get("li_at")
                self.headers["Csrf-Token"] = cookies.get("JSESSIONID")
                self.download_entire_course()
            else:
                with Session() as s:
                    if not os.path.exists(f'{self.course_slug}'):
                        os.makedirs(f'{self.course_slug}')
                    site = s.get(config.login_url)
                    bs_content = bs(site.content, "html.parser")

                    csrf = bs_content.find(
                        'input', {'name': 'csrfToken'}).get("value")
                    loginCsrfParam = bs_content.find(
                        "input", {"name": "loginCsrfParam"}).get("value")
                    login_data = {"session_key": self.email, "session_password": self.password,
                                  "csrfToken": csrf, "loginCsrfParam": loginCsrfParam}

                    status = self.login(s, login_data)

                    if status is None:
                        print("\n")
                        click.echo(
                            click.style(f"Wrong credentials, try again", fg="red"))
                        sys.exit(0)
                    else:
                        self.download_entire_course()

        except requests.exceptions.ConnectionError:
            print("\n")
            click.echo(click.style(
                f"Failed to connect", fg="red"))

    @staticmethod
    def resume_failed_ownloads():
        """
            Resume failed downloads
        """
        current_files = [file for file in os.listdir() if ".mp4" in file]
        if len(current_files) > 0:
            for file in current_files:
                if os.stat(file).st_size == 0:
                    os.remove(file)
            print("\n")
            click.echo(click.style(f"Resuming download..", fg="red"))

    def download_entire_course(self):
        """
            Download a course
        """
        self.resume_failed_ownloads()
        print("\n")
        try:
            r = requests.get(config.course_url.format(
                self.course_slug), cookies=self.cookies, headers=self.headers)
            course_name = r.json()['elements'][0]['title']
            course_name = re.sub(r'[\\/*?:"<>|]', "", course_name)
            chapters = r.json()['elements'][0]['chapters']
            exercise_files = r.json()["elements"][0]["exerciseFileUrls"]
            count = 1

            for chapter in chapters:
                chapter_name = chapter["title"]
                videos = chapter["videos"]
                if not os.path.exists(f'{self.course_slug}/{clean_name(chapter_name)}'):
                    os.makedirs(
                        f'{self.course_slug}/{clean_name(chapter_name)}')
                for video in videos:

                    video_name = re.sub(r'[\\/*?:"<>|]', "",
                                        video['title'])
                    video_slug = video['slug']
                    video_url = config.video_url.format(
                        self.course_slug, self.video_format, video_slug)
                    page_data = requests.get(
                        video_url, cookies=self.cookies, headers=self.headers)
                    page_json = page_data.json()
                    try:
                        download_url = page_json['elements'][0]['selectedVideo']['url']['progressiveUrl']
                        try:
                            subtitles = page_json['elements'][0]['selectedVideo']['transcript']
                        except:
                            click.echo(click.style(
                                f"Subtitles not found", fg="red"))
                            subtitles = None
                        duration_in_ms = int(page_json['elements'][0]
                                             ['selectedVideo']['durationInSeconds']) * 1000

                        click.echo(
                            click.style(f"current: {count}", fg="red"))
                        click.echo(
                            click.style(f"format: {self.video_format}p", fg="red"))
                        current_files = []
                        for file in os.listdir(f"./{self.course_slug}/{clean_name(chapter_name)}"):
                            if file.endswith(".mp4") and "-" in file:
                                ff = re.split(
                                    "\d+-", file)[1].replace(".mp4", "")
                                current_files.append(ff)
                    except Exception as e:
                        if 'url' in str(e):
                            click.echo(
                                click.style(f"This video is locked, you probably need a premium account", fg="red"))
                        else:
                            click.echo(
                                click.style(f"Failed to download this video", fg="red"))
                    else:
                        if clean_name(video_name) not in current_files:
                            if subtitles is not None and self.caption:
                                click.echo(click.style(
                                    f"Fetching subtitles..", fg="green"))
                                subtitle_lines = subtitles['lines']
                                write_subtitles(
                                    count, subtitle_lines, video_name, self.course_slug, chapter_name, duration_in_ms)
                            download_video(download_url, count,
                                           video_name, chapter_name, self.course_slug)
                        else:
                            click.echo(
                                click.style(f"skipping: " +
                                            video_name + "\n", fg="green"))
                        count += 1
            if len(exercise_files) > 0:
                download_exercises(exercise_files, self.course_slug)
            print("\n" + "Finished, start learning! :)")

        except Exception as e:
            print(e)
            click.echo(
                click.style("Failed to connect, try again later", fg="red"))



from setuptools import setup
from os import path

current_dir = path.abspath(path.dirname(__file__))

with open(path.join(current_dir, "README.md"), "r") as f:
    readme = f.read()

setup(
    name="llvd",
    version="2.1.1",
    url="https://github.com/knowbee/llvd.git",
    author="Igwaneza Bruce",
    author_email="knowbeeinc@gmail.com",
    description="Linkedin Learning Video Downloader CLI Tool",
    long_description=readme,
    long_description_content_type="text/markdown",
    platforms="any",
    python_requires=">=3.6",
    packages=["llvd"],
    install_requires=[
        "beautifulsoup4==4.9.3",
        "certifi==2020.12.5",
        "chardet==4.0.0",
        "click==7.1.2",
        "idna==2.10",
        "requests==2.25.1",
        "soupsieve==2.1",
        "tqdm==4.55.0",
        "urllib3==1.26.2",
        "click_spinner== 0.1.10"
    ],
    entry_points={
        "console_scripts": ['llvd = llvd:main']
    },
    classifiers=[
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
    ],
)
